/*
# %M% %Y% %I%
# The latest update : %G% at %U%
#
#%Z% dmFileRead ver %I%
#%Z% Created by 
#%Z%
#%Z% Usage : dmFileRead 
#%Z% Attention
#%Z%
*/
static char __sccs_id[] = "%Z%dmFileRead ver%I%; Date:%D% %Z%";
#define DEBUG
#include "genUtil.h"
#include "Memory.h"
#include "../inc/dmFile.h"

void
dmFileRead(FILE* fpt, dmFile* dm, char* message, int mode)
{
	int i, j;
	dmFile3Tag* dmTag;

	dmFileHeaderTailRead(fpt, dm, message, mode);

	switch(dm->version) {
		case 3: {
			// Header
			dmFile3HeaderRead(fpt, dm, message, mode);
			// Root Tag
			dmFile3TagDirectoryHeaderRead(fpt, &(dm->RootTag3), message, mode);
			for(i=0; i<dm->RootTag3.numberOfTags; i++) {
				dmTag = &(dm->RootTag3.Tags[i]);
				DEBUGPRINT2("Tags-i: %d/%d\n", i, dm->RootTag3.numberOfTags);
				dmFile3TagHeaderRead(fpt, &(dm->RootTag3.Tags[i]), message, mode);
				switch(dm->RootTag3.Tags[i].tagID) {
					case 0x15: { // tag
						DEBUGPRINT("TAG\n");
						dmFile3TagRead(fpt, &(dm->RootTag3.Tags[i]), message, mode);
						break;
					}
					case 0x14: { // tag directories
						DEBUGPRINT("TAG DIRECTORIES\n");	
						dm->RootTag3.Tags[i].tagDir = memoryAllocate(sizeof(dmFile3TagDirectory), message);
						dmFile3TagDirectoryHeaderRead(fpt, dm->RootTag3.Tags[i].tagDir, message, mode);	
						DEBUGPRINT1("Number of Tages: %d\n", dm->RootTag3.Tags[i].tagDir->numberOfTags);
						for(j=0; j<dm->RootTag3.Tags[i].tagDir->numberOfTags; j++) {
							DEBUGPRINT1("Tags-j(in tag directories): %d\n", j);
							dmFile3TagHeaderRead(fpt, &(dm->RootTag3.Tags[i].tagDir->Tags[j]), message, mode);	
							dmFile3TagRead(fpt, &(dm->RootTag3.Tags[i].tagDir->Tags[j]), message, mode);	
						}
						break;
					}
					case 0x0: {  
						DEBUGPRINT("FILE END\n")
						return; 
						break;
					}
					default: {
						dmFile3TagRead(fpt, &(dm->RootTag3.Tags[i]), message, mode);
						fprintf(stderr, "Not supported tagID: %X\n", dm->RootTag3.Tags[i].tagID);
						//exit(EXIT_FAILURE);
						break;
					}
				}
			}
			break;
		}
		case 4: {
			// Header
			dmFile4HeaderRead(fpt, dm, message, mode);
			// Root Tag
			dmFile4TagDirectoryHeaderRead(fpt, &(dm->RootTag4), message, mode);
			break;
		}
		default: {
			fprintf(stderr, "Not supported version: %d\n", dm->version);
			break;
		}
	}
}


void
dmFileHeaderTailRead(FILE* fpt, dmFile* dm, char* message, int mode)
{
	DEBUGPRINT("dmFileHeaderTailRead\n");
	fseek(fpt, 0L, SEEK_SET);
	fread(&(dm->version), sizeof(dmFileInteger4), 1, fpt);
#ifdef II
	memoryByteSwap(&(dm->version), sizeof(dmFileInteger4), 1);
#endif
	DEBUGPRINT1("version: %d\n", dm->version);

	fseek(fpt, -8L, SEEK_END);
	fread(&(dm->Tail), sizeof(dmFileInteger8), 1, fpt);
	if(dm->Tail!=0 && 3<=dm->version) {
		fprintf(stderr, "This file is not dm3/dm4: version %d \n", dm->version);
		exit(EXIT_FAILURE);
	}
}

void
dmFile3HeaderRead(FILE* fpt, dmFile* dm, char* message, int mode)
{
	DEBUGPRINT("dmFile3HeaderRead\n");
	// Header
	fseek(fpt, 0L, SEEK_SET);
	if(12!=sizeof(dm->Header3)) {
		fprintf(stderr, "Size is strange: %lu expected 12\n", sizeof(dm->Header4));
		fread(&(dm->Header3.version), sizeof(dm->Header3.version), 1, fpt);
		fread(&(dm->Header3.rootLength), sizeof(dm->Header3.rootLength), 1, fpt);
		fread(&(dm->Header3.byteOrder), sizeof(dm->Header3.byteOrder), 1, fpt);
	} else {
		DEBUGPRINT1("Header3 Size: %lu expect 12\n", sizeof(dm->Header4));
		fread(&(dm->Header3), sizeof(dm->Header3), 1, fpt);
	}
#ifdef II
	memoryByteSwap(&(dm->Header3.version), sizeof(dmFileInteger4), 1);
	memoryByteSwap(&(dm->Header3.rootLength), sizeof(dmFileInteger4), 1);
	memoryByteSwap(&(dm->Header3.byteOrder), sizeof(dmFileInteger4), 1);
#endif
	DEBUGPRINT1("version:    %d\n", dm->Header3.version);
	DEBUGPRINT1("RootLength: %d\n", dm->Header3.rootLength);
	DEBUGPRINT1("ByteOrder:  %d\n", dm->Header3.byteOrder);
}


void
dmFile4HeaderRead(FILE* fpt, dmFile* dm, char* message, int mode)
{
	// Header
	fseek(fpt, 0L, SEEK_SET);
	if(16!=sizeof(dm->Header4)) {
		fprintf(stderr, "Size is strange: %lu expected 16\n", sizeof(dm->Header4));
		fread(&(dm->Header4.version), sizeof(dm->Header4.version), 1, fpt);
		fread(&(dm->Header4.rootLength), sizeof(dm->Header4.rootLength), 1, fpt);
		fread(&(dm->Header4.byteOrder), sizeof(dm->Header4.byteOrder), 1, fpt);
	} else {
		DEBUGPRINT1("Header4 Size: %lu expect 16\n", sizeof(dm->Header4));
		fread(&(dm->Header4), sizeof(dm->Header4), 1, fpt);
	}
#ifdef II
	memoryByteSwap(&(dm->Header4.version), sizeof(dmFileInteger4), 1);
	memoryByteSwap(&(dm->Header4.rootLength), sizeof(dmFileInteger8), 1);
	memoryByteSwap(&(dm->Header4.byteOrder), sizeof(dmFileInteger4), 1);
#endif
	DEBUGPRINT1("version:    %d\n", dm->Header4.version);
	DEBUGPRINT1("RootLength: %lld\n", dm->Header4.rootLength);
	DEBUGPRINT1("ByteOrder:  %d\n", dm->Header4.byteOrder);
}

void
dmFile3TagDirectoryHeaderRead(FILE* fpt, dmFile3TagDirectory* RootTag3, char* message, int mode)
{
	DEBUGPRINT("dmFileTag3RootTagRead\n");
	// Root Tag
	fread(&(RootTag3->sortf), sizeof(dmFileInteger1), 1, fpt);
	fread(&(RootTag3->closef), sizeof(dmFileInteger1), 1, fpt);
	fread(&(RootTag3->numberOfTags), sizeof(dmFileInteger4), 1, fpt);
#ifdef II
	memoryByteSwap(&(RootTag3->numberOfTags), sizeof(dmFileInteger4), 1);
#endif
	DEBUGPRINT1("sortf:    %d\n", RootTag3->sortf);
	DEBUGPRINT1("closef:   %d\n", RootTag3->closef);
	DEBUGPRINT1("numberOfTags:   %d\n", RootTag3->numberOfTags);
	RootTag3->Tags = (dmFile3Tag*)memoryAllocate(sizeof(dmFile3Tag)*RootTag3->numberOfTags, "in dmFileRead");
}

void
dmFile4TagDirectoryHeaderRead(FILE* fpt, dmFile4TagDirectory* RootTag4, char* message, int mode)
{
	DBUGPRINT("dmFile4TagDirectoryHeaderRead");
	// Root Tag
	fread(&(RootTag4->sortf), sizeof(dmFileInteger1), 1, fpt);
	fread(&(RootTag4->closef), sizeof(dmFileInteger1), 1, fpt);
	fread(&(RootTag4->numberOfTags), sizeof(dmFileInteger8), 1, fpt);
#ifdef II
	memoryByteSwap(&(RootTag4->numberOfTags), sizeof(dmFileInteger8), 1);
#endif
	DEBUGPRINT1("sortf:    %d\n", RootTag4->sortf);
	DEBUGPRINT1("closef:   %d\n", RootTag4->closef);
	DEBUGPRINT1("numberOfTags:   %lld\n", RootTag4->numberOfTags);
	RootTag4->Tags = (dmFile4Tag*)memoryAllocate(sizeof(dmFile4Tag)*RootTag4->numberOfTags, "in dmFileRead");
}

void
dmFile3TagHeaderRead(FILE* fpt, dmFile3Tag* dmTag, char* message, int mode)
{
	DEBUGPRINT("dmFileTag3HeaderRead\n");	
	fread(&(dmTag->tagID), sizeof(dmFileInteger1), 1, fpt);
	fread(&(dmTag->tagNameLength), sizeof(dmFileInteger2), 1, fpt);
#ifdef II
	memoryByteSwap(&(dmTag->tagNameLength), sizeof(dmFileInteger2), 1);		
#endif
	if(0<dmTag->tagNameLength) {
		dmTag->tagName = (char*)memoryAllocate(sizeof(char)*(dmTag->tagNameLength+1), "in dmFileRead");
		fread(dmTag->tagName, sizeof(char), dmTag->tagNameLength, fpt); 
	}
	DEBUGPRINT3("tagID(hex) %X tagNameLength %d tagName %s\n", dmTag->tagID, dmTag->tagNameLength, dmTag->tagName);
}

void
dmFile3TagRead(FILE* fpt, dmFile3Tag* dmTag, char* message, int mode)
{
	int j;
	DEBUGPRINT("dmFileTag3Read\n");	
	fread(dmTag->seperator, sizeof(char), 4, fpt); 
	DEBUGPRINT1("%s\n",dmTag->seperator);
	if(0!=strncmp(dmTag->seperator, "%%%%", 4)) {
		fprintf(stderr, "Not supported : Strange!!!\n");
	}
	fread(&(dmTag->ninfo), sizeof(dmFileInteger4), 1, fpt); 
#ifdef II
	memoryByteSwap(&(dmTag->ninfo), sizeof(dmFileInteger4), 1);		
#endif
	DEBUGPRINT1("tag ninfo %d\n", dmTag->ninfo);
	if(0==dmTag->ninfo) {
		DEBUGPRINT1("tag ninfo is zero: %d\n", dmTag->ninfo);
		return;
	}
	dmTag->info = (dmFileInteger4*)memoryAllocate(sizeof(dmFileInteger4)*dmTag->ninfo, "in dmFileRead");					
	fread(dmTag->info, sizeof(dmFileInteger4), dmTag->ninfo, fpt);
#ifdef II
	memoryByteSwap(dmTag->info, sizeof(dmFileInteger4), dmTag->ninfo);		
#endif
	DEBUGPRINT2("tag: info[0] %X ninfo %d\n", dmTag->info[0], dmTag->ninfo);
	switch(dmTag->info[0]) {
		case dmFileTagDataTypeStruct: {
			DEBUGPRINT1("Struct: numEntry %d\n", dmTag->info[2]);
			// dmTag->info[2]       ; num of Entries 
			// dmTag->info[2*n + 4] ; data type of value-n
			dmTag->value2 = (void*)memoryAllocate(sizeof(void*)*dmTag->info[2], "in dmFileRead");
			for(j=0; j<dmTag->info[2]; j++) {
				DEBUGPRINT2("value %d type %d\n", j, dmTag->info[2*j+4]); 
				dmTag->value2[j] = memoryAllocate(__dmFileTagDataTypeSize[dmTag->info[2*j+4]], "in dmFileRead");
				fread(dmTag->value2[j], __dmFileTagDataTypeSize[dmTag->info[2*j+4]], 1, fpt);	
#ifdef II
				memoryByteSwap(dmTag->value2[j], __dmFileTagDataTypeSize[dmTag->info[2*j+4]], 1);		
#endif 
#ifdef DEBUG
				dmFileValuePrint(stderr, dmTag->value2[j], __dmFileTagDataTypeSize[dmTag->info[2*j+4]], 1); 
#endif
			}
			break;
		}
		case dmFileTagDataTypeArray: {
			DEBUGPRINT1("ARRAY %d\n", dmTag->info[0]);
			// dmTag->info[0] ; data type 
			// dmTag->info[1] ; data array size 
			if(0<dmTag->info[1]) {
				dmTag->value = memoryAllocate(__dmFileTagDataTypeSize[dmTag->info[0]]*dmTag->info[1], "in dmFileRead");
				fread(dmTag->value, __dmFileTagDataTypeSize[dmTag->info[0]], dmTag->info[1], fpt);
#ifdef DEBUG
				dmFileValuePrint(stderr, dmTag->value, __dmFileTagDataTypeSize[dmTag->info[0]], dmTag->info[1]); 
#endif
			} else {
				fprintf(stderr, "Not supported data type for array: %d\n", dmTag->info[1]);
			}
			break;
		}
		case dmFileTagDataTypeString: {
			//dmTag->value = memoryAllocate(__dmFileTagDataTypeSize[dmTag->info[0]], "in dmFileRead");
			//fread(dmTag->value, __dmFileTagDataTypeSize[dmTag->info[0]], 1, fpt);
			//dmFileValuePrint(stderr, dmTag->value, __dmFileTagDataTypeSize[dmTag->info[0]], 1);
			//fprintf(stderr, "Not supported tagType: %d\n", dmTag->info[j]);
			break;
		}	
		default: {
			dmTag->value = memoryAllocate(__dmFileTagDataTypeSize[dmTag->info[0]], "in dmFileRead");
			fread(dmTag->value, __dmFileTagDataTypeSize[dmTag->info[0]], 1, fpt);
			dmFileValuePrint(stderr, dmTag->value, __dmFileTagDataTypeSize[dmTag->info[0]], 1); 
			//fprintf(stderr, "Not supported tagType: %d\n", dmTag->info[j]);
			break;
		}
	}
}

